from qtpy import QtCore, QtGui, QtWidgets

import examples.dayu._mock_data as mock
from qtui.dayu import utils
from qtui.dayu.item_model import MSortFilterModel, MTableModel
from qtui.dayu.item_view import MTableView
from qtui.dayu.theme import MTheme
from qtui.dayu.types import HeaderData, ModelData


class MPushButtonDelegate(QtWidgets.QStyledItemDelegate):
    sig_clicked = QtCore.Signal(object)

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.editor = None
        self.showed = False
        self.exclusive = True
        self.parent_widget = None

    def editorEvent(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        event: QtCore.QEvent,
        model: QtCore.QAbstractItemModel,
        option: QtWidgets.QStyleOptionViewItem,
        index: QtCore.QModelIndex,
    ):
        if event.type() == QtCore.QEvent.Type.MouseButtonRelease:
            index = utils.real_index(index)
            self.sig_clicked.emit(index.internalPointer())
            return True
        return False

    def paint(  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        painter: QtGui.QPainter,
        option: QtWidgets.QStyleOptionViewItem,
        index: QtCore.QModelIndex,
    ):
        button = QtWidgets.QStyleOptionButton()
        button.rect = option.rect  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
        button.text = (  # pyright: ignore[reportAttributeAccessIssue]
            "Click Me (" + str(index.data(QtCore.Qt.ItemDataRole.DisplayRole)) + ")"
        )
        button.state = QtWidgets.QStyle.StateFlag.State_Enabled  # pyright: ignore[reportAttributeAccessIssue]

        QtWidgets.QApplication.style().drawControl(
            QtWidgets.QStyle.ControlElement.CE_PushButton, button, painter
        )


header_list: list[HeaderData] = [
    {
        "label": "Name",
        "key": "name",
        "checkable": True,
        "searchable": True,
        "width": 200,
        "font": mock.font_underline,
        "icon": "user_fill.svg",
    },
    {
        "label": "Sex",
        "key": "sex",
        "searchable": True,
        "selectable": True,
        "icon": mock.sex_icon_color,
    },
    {
        "label": "Age",
        "key": "age",
        "width": 90,
        "searchable": True,
        "editable": True,
        "display": mock.age_display,
        "font": mock.font_bold,
    },
    {
        "label": "Address",
        "key": "city",
        "selectable": True,
        "searchable": True,
        "exclusive": False,
        "width": 120,
        "display": mock.city_display,
        "bg_color": mock.city_bg_color,
    },
    {
        "label": "Score",
        "key": "score",
    },
]


class DelegateButtonExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        model_1 = MTableModel()
        model_1.set_header_list(header_list)
        model_sort = MSortFilterModel()
        model_sort.setSourceModel(model_1)

        table_grid = MTableView(size=MTheme().small, show_row_count=True)
        table_grid.setShowGrid(True)
        table_grid.setModel(model_sort)
        model_sort.set_header_list(header_list)
        table_grid.set_header_list(header_list)
        button_delegate = MPushButtonDelegate(parent=self)
        table_grid.setItemDelegateForColumn(4, button_delegate)
        button_delegate.sig_clicked.connect(self.slot_cell_clicked)
        model_1.set_data_list(
            [
                {
                    "name": "John Brown",
                    "sex": "Male",
                    "sex_list": ["Male", "Female"],
                    "age": 18,
                    "score": 89,
                    "city": "New York",
                    "city_list": ["New York", "Ottawa", "London", "Sydney"],
                    "date": "2016-10-03",
                },
                {
                    "name": "Jim Green",
                    "sex": "Male",
                    "sex_list": ["Male", "Female"],
                    "age": 24,
                    "score": 55,
                    "city": "London",
                    "city_list": ["New York", "Ottawa", "London", "Sydney"],
                    "date": "2016-10-01",
                },
            ]
        )

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(table_grid)
        self.setLayout(main_lay)

    def slot_cell_clicked(self, data_dict: ModelData):
        print(data_dict)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = DelegateButtonExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtCore, QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.progress_bar import MProgressBar
from qtui.dayu.push_button import MPushButton


class ProgressBarExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MProgressBar")
        self._init_ui()

    def _init_ui(self):
        progress_1 = MProgressBar()
        progress_1.setValue(10)
        progress_1.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        progress_2 = MProgressBar()
        progress_2.setValue(80)

        progress_normal = MProgressBar()
        progress_normal.setValue(30)
        progress_success = MProgressBar().success()
        progress_success.setValue(100)
        progress_error = MProgressBar().error()
        progress_error.setValue(50)
        form_lay = QtWidgets.QFormLayout()
        form_lay.addRow("Primary:", progress_normal)
        form_lay.addRow("Success:", progress_success)
        form_lay.addRow("Error:", progress_error)

        self.progress_count = 0
        self.timer = QtCore.QTimer()
        self.timer.setInterval(10)
        self.timer.timeout.connect(self.slot_timeout)
        run_button = MPushButton(text="Run Something")
        run_button.clicked.connect(self.slot_run)
        self.auto_color_progress = MProgressBar().auto_color()
        auto_color_lay = QtWidgets.QVBoxLayout()
        auto_color_lay.addWidget(run_button)
        auto_color_lay.addWidget(self.auto_color_progress)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Basic"))

        main_lay.addWidget(progress_1)
        main_lay.addWidget(progress_2)
        main_lay.addWidget(MDivider("different type"))
        main_lay.addLayout(form_lay)
        main_lay.addWidget(MDivider("auto color"))
        main_lay.addLayout(auto_color_lay)
        main_lay.addStretch()
        self.setLayout(main_lay)

    def slot_run(self):
        self.timer.start()
        self.auto_color_progress.setValue(0)

    def slot_timeout(self):
        if self.auto_color_progress.value() > 99:
            self.timer.stop()
        else:
            self.auto_color_progress.setValue(self.auto_color_progress.value() + 1)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ProgressBarExample()
        MTheme().apply(test)
        test.show()



import functools
from collections.abc import Callable
from typing import Any

from qtpy import QtCore, QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.push_button import MPushButton
from qtui.dayu.toast import MToast


class MWorkThread(QtCore.QThread):
    def __init__(self, parent: QtCore.QObject | None = None):
        super().__init__(parent)

    def run(self):
        import time

        time.sleep(3)


class ToastExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MToast")
        self._init_ui()

    def _init_ui(self):
        button3 = MPushButton(text="Normal Message").primary()
        button4 = MPushButton(text="Success Message").success()
        button5 = MPushButton(text="Warning Message").warning()
        button6 = MPushButton(text="Error Message").danger()
        button3.clicked.connect(
            functools.partial(self.slot_show_message, MToast.info, {"text": "No use"})
        )
        button4.clicked.connect(
            functools.partial(
                self.slot_show_message, MToast.success, {"text": "Success"}
            )
        )
        button5.clicked.connect(
            functools.partial(
                self.slot_show_message, MToast.warning, {"text": "Not supported"}
            )
        )
        button6.clicked.connect(
            functools.partial(
                self.slot_show_message,
                MToast.error,
                {"text": "Payment failed, please try again"},
            )
        )

        sub_lay1 = QtWidgets.QHBoxLayout()
        sub_lay1.addWidget(button3)
        sub_lay1.addWidget(button4)
        sub_lay1.addWidget(button5)
        sub_lay1.addWidget(button6)

        loading_button = MPushButton("Loading Toast").primary()
        loading_button.clicked.connect(self.slot_show_loading)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different type"))
        main_lay.addLayout(sub_lay1)
        main_lay.addWidget(
            MLabel(
                "Different prompt status: success, failure, loading. Default 2 seconds to disappear"  # noqa: E501
            )
        )
        main_lay.addWidget(loading_button)

        main_lay.addStretch()
        self.setLayout(main_lay)

    def slot_show_message(self, func: Callable[..., Any], config: dict[str, Any]):
        func(parent=self, **config)

    def slot_set_config(self, func: Callable[..., Any], config: dict[str, Any]):
        func(**config)

    def slot_show_loading(self):
        my_thread = MWorkThread(parent=self)
        my_thread.finished.connect(self.slot_finished)
        my_thread.start()
        self.msg = MToast.loading("Loading", parent=self)

    def slot_finished(self):
        self.msg.close()
        MToast.success("Loading successful", self)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ToastExample()
        MTheme().apply(test)
        test.show()



"""MLineTabWidget"""

from __future__ import annotations

from qtpy import QtCore, QtGui, QtWidgets

from .button_group import MButtonGroupBase, MButtonGroupData
from .divider import MDivider
from .stacked_widget import MStackedWidget
from .theme import MTheme
from .tool_button import MToolButton


class MUnderlineButton(MToolButton):
    """MUnderlineButton"""

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setCheckable(True)


class MUnderlineButtonGroup(MButtonGroupBase[MUnderlineButton]):
    """MUnderlineButtonGroup"""

    sig_checked_changed = QtCore.Signal(int)

    def __init__(self, tab: MLineTabWidget, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent=parent)
        self._line_tab = tab
        self.set_spacing(1)
        self._button_group.setExclusive(True)
        self._button_group.buttonClicked.connect(self.sig_checked_changed.emit)

    def create_button(self, data_dict: MButtonGroupData):
        button = MUnderlineButton(parent=self)
        if data_dict.get("svg"):
            button.svg(data_dict.get("svg", ""))
        if data_dict.get("text"):
            if data_dict.get("svg") or data_dict.get("icon"):
                button.text_beside_icon()
            else:
                button.text_only()
        else:
            button.icon_only()
        button.set_dayu_size(self._line_tab.get_dayu_size())
        return button

    def update_size(self, size: int):
        for button in self.get_buttons():
            button.set_dayu_size(size)

    def set_dayu_checked(self, value: int):
        """Set current checked button's id"""
        button = self._button_group.button(value)
        button.setChecked(True)
        self.sig_checked_changed.emit(value)

    def get_dayu_checked(self):
        """Get current checked button's id"""
        return self._button_group.checkedId()

    dayu_checked = QtCore.Property(
        int,
        get_dayu_checked,
        set_dayu_checked,
        notify=sig_checked_changed,
    )


class MLineTabWidget(QtWidgets.QWidget):
    """MLineTabWidget"""

    def __init__(
        self,
        alignment: QtCore.Qt.AlignmentFlag = QtCore.Qt.AlignmentFlag.AlignCenter,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent=parent)
        self.tool_button_group = MUnderlineButtonGroup(tab=self)
        self.bar_layout = QtWidgets.QHBoxLayout()
        self.bar_layout.setContentsMargins(0, 0, 0, 0)
        if alignment == QtCore.Qt.AlignmentFlag.AlignCenter:
            self.bar_layout.addStretch()
            self.bar_layout.addWidget(self.tool_button_group)
            self.bar_layout.addStretch()
        elif alignment == QtCore.Qt.AlignmentFlag.AlignLeft:
            self.bar_layout.addWidget(self.tool_button_group)
            self.bar_layout.addStretch()
        elif alignment == QtCore.Qt.AlignmentFlag.AlignRight:
            self.bar_layout.addStretch()
            self.bar_layout.addWidget(self.tool_button_group)
        self.stack_widget = MStackedWidget()
        self.tool_button_group.sig_checked_changed.connect(
            self.stack_widget.setCurrentIndex
        )
        main_lay = QtWidgets.QVBoxLayout()
        main_lay.setContentsMargins(0, 0, 0, 0)
        main_lay.setSpacing(0)
        main_lay.addLayout(self.bar_layout)
        main_lay.addWidget(MDivider())
        main_lay.addSpacing(5)
        main_lay.addWidget(self.stack_widget)
        self.setLayout(main_lay)
        self._dayu_size = MTheme().default_size

    def append_widget(self, widget: QtWidgets.QWidget):
        """Add the widget to line tab's right position."""
        self.bar_layout.addWidget(widget)

    def insert_widget(self, widget: QtWidgets.QWidget):
        """Insert the widget to line tab's left position."""
        self.bar_layout.insertWidget(0, widget)

    def add_tab(
        self, widget: QtWidgets.QWidget, data_dict: MButtonGroupData | str | QtGui.QIcon
    ):
        """Add a tab"""
        self.stack_widget.addWidget(widget)
        self.tool_button_group.add_button(data_dict, self.stack_widget.count() - 1)

    def get_dayu_size(self) -> int:
        """
        Get the line tab size.
        :return: integer
        """
        return self._dayu_size

    def set_dayu_size(self, value: int):
        """
        Set the line tab size.
        :param value: integer
        :return: None
        """
        self._dayu_size = value
        self.tool_button_group.update_size(self._dayu_size)
        self.style().polish(self)

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)



from qtpy import QtCore, QtWidgets

from qtui.dayu.avatar import MAvatar
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MPixmap
from qtui.dayu.theme import MTheme


class AvatarExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Example for MAvatar")
        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different size"))

        size_list = [
            ("Huge", MAvatar.huge),
            ("Large", MAvatar.large),
            ("Medium", MAvatar.medium),
            ("Small", MAvatar.small),
            ("Tiny", MAvatar.tiny),
        ]

        self.pix_map_list = [
            None,
            MPixmap("avatar.png"),
            MPixmap("app-maya.png"),
            MPixmap("app-nuke.png"),
            MPixmap("app-houdini.png"),
        ]
        form_lay = QtWidgets.QFormLayout()
        form_lay.setLabelAlignment(QtCore.Qt.AlignmentFlag.AlignRight)

        for label, cls in size_list:
            h_lay = QtWidgets.QHBoxLayout()
            for image in self.pix_map_list:
                avatar_tmp = cls(image)
                h_lay.addWidget(avatar_tmp)
            h_lay.addStretch()
            form_lay.addRow(MLabel(label), h_lay)
        main_lay.addLayout(form_lay)
        self.register_field("image", None)
        main_lay.addWidget(MDivider("different image"))
        avatar = MAvatar()
        self.bind("image", avatar, "dayu_image")
        button = MPushButton(text="Change Avatar Image").primary()
        button.clicked.connect(self.slot_change_image)

        main_lay.addWidget(avatar)
        main_lay.addWidget(button)
        main_lay.addStretch()
        self.setLayout(main_lay)

    def slot_change_image(self):
        """Set the Avatar image random by data bind."""
        import random

        self.set_field("image", random.choice(self.pix_map_list))


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = AvatarExample()
        MTheme().apply(test)
        test.show()



"""MTheme"""

from __future__ import annotations

import string
from enum import StrEnum
from pathlib import Path
from typing import Any

from qtpy import QtGui, QtWidgets

from .qt import get_scale_factor
from .utils import fade_color, generate_color, get_static_file, singleton

DEFAULT_STATIC_DIR = Path(__file__).parent / "static"


class QssTemplate(string.Template):
    delimiter = "@"
    idpattern = r"[_a-z][_a-z0-9]*"


@singleton
class MTheme:
    class ThemeType(StrEnum):
        Light = "light"
        Dark = "dark"
        System = "system"

    class GlobalColor(StrEnum):
        Blue = "#1890ff"
        Purple = "#722ed1"
        Cyan = "#13c2c2"
        Green = "#52c41a"
        Magenta = "#eb2f96"
        Pink = "#ef5b97"
        Red = "#f5222d"
        Orange = "#fa8c16"
        Yellow = "#fadb14"
        Volcano = "#fa541c"
        Geekblue = "#2f54eb"
        Lime = "#a0d911"
        Gold = "#faad14"
        Female = "#ef5b97"
        Male = "#4ebbff"

    def __init__(
        self, theme: ThemeType = ThemeType.Dark, primary_color: str = "orange"
    ):
        # if not self._initialized:
        super().__init__()
        default_qss_file = get_static_file("main.qss")
        self.default_qss = QssTemplate(default_qss_file.read_text())
        self._init_sizes()
        self._init_font()
        self._init_semantic_color()
        self.set_primary_color(primary_color or MTheme.GlobalColor.Blue)
        self.set_theme(theme)
        self.unit = "px"
        self.font_unit = "pt"

    def _init_font(self):
        """
        Init font size and family
        """

        self.font_family = f"{QtGui.QFont().families()}"
        self.font_size_base = 11
        self.font_size_large = self.font_size_base + 2
        self.font_size_small = self.font_size_base - 2
        self.h1_size = int(self.font_size_base * 2.71)
        self.h2_size = int(self.font_size_base * 2.12)
        self.h3_size = int(self.font_size_base * 1.71)
        self.h4_size = int(self.font_size_base * 1.41)

    def _init_semantic_color(self):
        """
        Init semantic color
        """
        self.info_color = MTheme.GlobalColor.Blue
        self.success_color = MTheme.GlobalColor.Green
        self.processing_color = MTheme.GlobalColor.Blue
        self.error_color = MTheme.GlobalColor.Red
        self.warning_color = MTheme.GlobalColor.Gold

        self.info_1 = fade_color(self.info_color, "15%")
        self.info_2 = generate_color(self.info_color, 2)
        self.info_3 = fade_color(self.info_color, "35%")
        self.info_4 = generate_color(self.info_color, 4)
        self.info_5 = generate_color(self.info_color, 5)
        self.info_6 = generate_color(self.info_color, 6)
        self.info_7 = generate_color(self.info_color, 7)
        self.info_8 = generate_color(self.info_color, 8)
        self.info_9 = generate_color(self.info_color, 9)
        self.info_10 = generate_color(self.info_color, 10)

        self.success_1 = fade_color(self.success_color, "15%")
        self.success_2 = generate_color(self.success_color, 2)
        self.success_3 = fade_color(self.success_color, "35%")
        self.success_4 = generate_color(self.success_color, 4)
        self.success_5 = generate_color(self.success_color, 5)
        self.success_6 = generate_color(self.success_color, 6)
        self.success_7 = generate_color(self.success_color, 7)
        self.success_8 = generate_color(self.success_color, 8)
        self.success_9 = generate_color(self.success_color, 9)
        self.success_10 = generate_color(self.success_color, 10)

        self.warning_1 = fade_color(self.warning_color, "15%")
        self.warning_2 = generate_color(self.warning_color, 2)
        self.warning_3 = fade_color(self.warning_color, "35%")
        self.warning_4 = generate_color(self.warning_color, 4)
        self.warning_5 = generate_color(self.warning_color, 5)
        self.warning_6 = generate_color(self.warning_color, 6)
        self.warning_7 = generate_color(self.warning_color, 7)
        self.warning_8 = generate_color(self.warning_color, 8)
        self.warning_9 = generate_color(self.warning_color, 9)
        self.warning_10 = generate_color(self.warning_color, 10)

        self.error_1 = fade_color(self.error_color, "15%")
        self.error_2 = generate_color(self.error_color, 2)
        self.error_3 = fade_color(self.error_color, "35%")
        self.error_4 = generate_color(self.error_color, 4)
        self.error_5 = generate_color(self.error_color, 5)
        self.error_6 = generate_color(self.error_color, 6)
        self.error_7 = generate_color(self.error_color, 7)
        self.error_8 = generate_color(self.error_color, 8)
        self.error_9 = generate_color(self.error_color, 9)
        self.error_10 = generate_color(self.error_color, 10)

        self.text_error_color = self.error_7
        self.text_color_inverse = "#fff"
        self.text_warning_color = self.warning_7

    def _init_sizes(self):
        """
        Init sizes
        """
        scale_factor_x, _ = get_scale_factor()
        self.border_radius_large = int(6 * scale_factor_x)
        self.border_radius_base = int(4 * scale_factor_x)
        self.border_radius_small = int(2 * scale_factor_x)
        self.huge = int(48 * scale_factor_x)
        self.large = int(40 * scale_factor_x)
        self.medium = int(32 * scale_factor_x)
        self.small = int(24 * scale_factor_x)
        self.tiny = int(18 * scale_factor_x)
        self.huge_icon = int((48 - 20) * scale_factor_x)
        self.large_icon = int((40 - 16) * scale_factor_x)
        self.medium_icon = int((32 - 12) * scale_factor_x)
        self.small_icon = int((24 - 10) * scale_factor_x)
        self.tiny_icon = int((18 - 8) * scale_factor_x)
        self.default_size = int(32 * scale_factor_x)
        self.badge_width_radius = int(8 * scale_factor_x)
        self.badge_width = int(16 * scale_factor_x)
        self.badge_dot = int(8 * scale_factor_x)
        self.badge_dot_radius = int(4 * scale_factor_x)
        self.drop_down_huge = int(20 * scale_factor_x)
        self.drop_down_large = int(16 * scale_factor_x)
        self.drop_down_medium = int(14 * scale_factor_x)
        self.drop_down_small = int(10 * scale_factor_x)
        self.drop_down_tiny = int(8 * scale_factor_x)
        self.spin_box_huge = int(28 * scale_factor_x)
        self.spin_box_large = int(26 * scale_factor_x)
        self.spin_box_medium = int(24 * scale_factor_x)
        self.spin_box_small = int(20 * scale_factor_x)
        self.spin_box_tiny = int(18 * scale_factor_x)
        self.spin_box_icon_huge = int(14 * scale_factor_x)
        self.spin_box_icon_large = int(12 * scale_factor_x)
        self.spin_box_icon_medium = int(10 * scale_factor_x)
        self.spin_box_icon_small = int(8 * scale_factor_x)
        self.spin_box_icon_tiny = int(6 * scale_factor_x)
        self.drag_border = int(2 * scale_factor_x)
        self.drag_border_radius = int(10 * scale_factor_x)
        self.drag_padding_x = int(20 * scale_factor_x)
        self.drag_padding_y = int(40 * scale_factor_x)
        self.drag_size = int(60 * scale_factor_x)
        self.switch_width_huge = int(58 * scale_factor_x)
        self.switch_height_huge = int(30 * scale_factor_x)
        self.switch_radius_huge = int(15 * scale_factor_x)
        self.switch_width_large = int(48 * scale_factor_x)
        self.switch_height_large = int(24 * scale_factor_x)
        self.switch_radius_large = int(12 * scale_factor_x)
        self.switch_width_medium = int(38 * scale_factor_x)
        self.switch_height_medium = int(19 * scale_factor_x)
        self.switch_radius_medium = int(9 * scale_factor_x)
        self.switch_width_small = int(28 * scale_factor_x)
        self.switch_height_small = int(14 * scale_factor_x)
        self.switch_radius_small = int(7 * scale_factor_x)
        self.switch_width_tiny = int(18 * scale_factor_x)
        self.switch_height_tiny = int(10 * scale_factor_x)
        self.switch_radius_tiny = int(5 * scale_factor_x)
        self.check_box_size = int(13 * scale_factor_x)
        self.check_box_spacing = int(4 * scale_factor_x)
        self.radio_size = int(14 * scale_factor_x)
        self.radio_radius = int(14 * scale_factor_x / 2.0)
        self.radio_spacing = int(4 * scale_factor_x)
        self.slider_height = int(4 * scale_factor_x)
        self.slider_radius = int(3 * scale_factor_x)
        self.slider_handle_size = int(8 * scale_factor_x)
        self.slider_handle_radius = int(8 * scale_factor_x / 1.5)
        self.progress_circle_default_radius = int(120 * scale_factor_x)
        self.progress_bar_size = int(12 * scale_factor_x)
        self.progress_bar_radius = int(12 * scale_factor_x / 2.0)
        self.toast_size = int(120 * scale_factor_x)
        self.toast_icon_size = int(60 * scale_factor_x)
        self.big_view_default_size = int(120 * scale_factor_x)
        self.big_view_max_size = int(400 * scale_factor_x)
        self.big_view_min_size = int(24 * scale_factor_x)
        self.indicator_padding = int(4 * scale_factor_x)
        self.indicator_size = int(8 * scale_factor_x)
        self.scroll_bar_size = int(12 * scale_factor_x)
        self.scroll_bar_min_length = int(20 * scale_factor_x)
        self.scroll_bar_margin = int(12 * scale_factor_x * 2) + 1
        self.scroll_bar_radius = int(12 * scale_factor_x / 2.0)

    def set_primary_color(self, color: str):
        self.primary_color = color
        self.primary_1 = generate_color(color, 1)
        self.primary_2 = generate_color(color, 2)
        self.primary_3 = generate_color(color, 3)
        self.primary_4 = generate_color(color, 4)
        self.primary_5 = generate_color(color, 5)
        self.primary_6 = generate_color(color, 6)
        self.primary_7 = generate_color(color, 7)
        self.primary_8 = generate_color(color, 8)
        self.primary_9 = generate_color(color, 9)
        self.primary_10 = generate_color(color, 10)
        self.item_hover_bg = self.primary_1
        self.hyperlink_style = f"""
        <style>
         a {{
            text-decoration: none;
            color: {self.primary_color};
        }}
        </style>"""

    def set_theme(self, theme: ThemeType):
        if theme == MTheme.ThemeType.Light:
            self._light()
        else:
            self._dark()
        self._set_icons(theme)

    def _set_icons(self, theme: ThemeType):
        """
        Set icons
        """
        pre_str = DEFAULT_STATIC_DIR.as_posix()
        suf_str = "" if theme == MTheme.ThemeType.Light else "_dark"
        url_prefix = f"{pre_str}/{{}}{suf_str}.png"
        url_prefix_2 = f"{pre_str}/{{}}.svg"
        self.icon_down = url_prefix.format("down_line")
        self.icon_up = url_prefix.format("up_line")
        self.icon_left = url_prefix.format("left_line")
        self.icon_right = url_prefix.format("right_line")
        self.icon_close = url_prefix.format("close_line")
        self.icon_calender = url_prefix.format("calendar_fill")
        self.icon_splitter = url_prefix.format("splitter")
        self.icon_float = url_prefix.format("float")
        self.icon_size_grip = url_prefix.format("size_grip")

        self.icon_check = url_prefix_2.format("check")
        self.icon_minus = url_prefix_2.format("minus")
        self.icon_circle = url_prefix_2.format("circle")
        self.icon_sphere = url_prefix_2.format("sphere")

    def _dark(self):
        self.title_color = "#ffffff"
        self.primary_text_color = "#d9d9d9"
        self.secondary_text_color = "#a6a6a6"
        self.disable_color = "#737373"
        self.border_color = "#1e1e1e"
        self.divider_color = "#262626"
        self.header_color = "#0a0a0a"
        self.icon_color = "#a6a6a6"

        self.background_color = "#323232"
        self.background_selected_color = "#292929"
        self.background_in_color = "#3a3a3a"
        self.background_out_color = "#494949"
        self.mask_color = fade_color(self.background_color, "90%")
        self.toast_color = "#555555"

    def _light(self):
        self.title_color = "#262626"
        self.primary_text_color = "#595959"
        self.secondary_text_color = "#8c8c8c"
        self.disable_color = "#e5e5e5"
        self.border_color = "#d9d9d9"
        self.divider_color = "#e8e8e8"
        self.header_color = "#fafafa"
        self.icon_color = "#8c8c8c"

        self.background_color = "#f8f8f9"
        self.background_selected_color = "#bfbfbf"
        self.background_in_color = "#ffffff"
        self.background_out_color = "#eeeeee"
        self.mask_color = fade_color(self.background_color, "90%")
        self.toast_color = "#333333"

    def get_color(self, prefix: str) -> str:
        return str(getattr(self, f"{prefix}_color"))

    def _theme_vals(self) -> dict[str, Any]:
        return {k: v for k, v in vars(self).items() if not k.startswith("_")}

    def apply(self, widget: QtWidgets.QWidget):
        widget.setStyleSheet(self.default_qss.substitute(self._theme_vals()))

    def change_theme(self, widget: QtWidgets.QWidget, theme: ThemeType, color: str):
        self.set_theme(theme)
        self.set_primary_color(color)
        self.apply(widget.window())
        for sub_widget in widget.window().findChildren(QtWidgets.QComboBox):
            self.apply(sub_widget)
        for sub_widget in widget.window().findChildren(QtWidgets.QSplitter):
            self.apply(sub_widget)

    def deco(self, cls: type[QtWidgets.QWidget]):
        original_init__ = cls.__init__

        def my__init__(instance: QtWidgets.QWidget, *args: Any, **kwargs: Any):
            original_init__(instance, *args, **kwargs)
            instance.setStyleSheet(self.default_qss.substitute(self._theme_vals()))

        def polish(instance: QtWidgets.QWidget):
            instance.style().polish(instance)

        cls.__init__ = my__init__
        cls.polish = polish  # pyright: ignore[reportAttributeAccessIssue]
        return cls



from qtpy import QtWidgets

from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.line_edit import MLineEdit
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MPixmap


class FieldMixinExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.register_field("my_name", "xiaoming")
        self.register_field("thumbnail_path", "")
        self.register_field("is_enable", True)
        self.register_field("status", "waiting")
        self.register_field("str_enable", self.computed_str_enable)
        self.register_field("thumbnail_pix_map", self.computed_thumbnail_pix_map)
        self.register_field("email", self.computed_email)

        name2_label = MLabel()
        email_label = MLabel()
        thumbnail_label = MLabel()
        enable_button = MPushButton().primary()
        self.bind("my_name", name2_label, "dayu_text")
        self.bind("email", email_label, "dayu_text")
        self.bind("is_enable", enable_button, "enabled")
        self.bind("thumbnail_pix_map", thumbnail_label, "pixmap")
        self.bind("str_enable", enable_button, "text")

        button = MPushButton(text="Change Data").primary()
        button.clicked.connect(self.slot_change_data)
        main_lay = QtWidgets.QGridLayout()
        main_lay.addWidget(MLabel("Avatar:"), 0, 0)
        main_lay.addWidget(thumbnail_label, 0, 1)
        main_lay.addWidget(MLabel("Name:"), 1, 0)
        main_lay.addWidget(
            self.bind("my_name", MLineEdit(), "text", signal="textEdited"), 1, 1
        )
        main_lay.addWidget(MLabel("Email:"), 2, 0)
        main_lay.addWidget(email_label, 2, 1)
        main_lay.addWidget(MLabel("Enabled:"), 3, 0)
        main_lay.addWidget(enable_button, 3, 1)
        main_lay.addWidget(button, 4, 1)

        temp_lay = QtWidgets.QVBoxLayout()
        temp_lay.addLayout(main_lay)
        temp_lay.addStretch()
        self.setLayout(temp_lay)

    def computed_str_enable(self):
        return "Enabled" if self.field("is_enable") else "Disabled"

    def computed_thumbnail_pix_map(self):
        return MPixmap(self.field("thumbnail_path"))

    def computed_email(self):
        return "{}@phenom-films.com".format(self.field("my_name"))

    def slot_change_data(self):
        import random

        self.set_field(
            "my_name",
            random.choice(["xiaoming", "xiaohua", "xiaohong", "hahaha", "lalalala"]),
        )
        self.set_field(
            "thumbnail_path",
            "app-{}.png".format(random.choice(["maya", "nuke", "houdini"])),
        )
        self.set_field("is_enable", bool(random.randint(0, 1)))
        self.set_field("status", "haha")


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = FieldMixinExample()
        MTheme().apply(test)
        test.show()



import functools

from qtpy import QtCore, QtWidgets

from qtui.dayu.button_group import MRadioButtonGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MIcon
from qtui.dayu.types import MButtonGroupData


class RadioButtonGroupExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        radio_group_h = MRadioButtonGroup()
        radio_group_h.set_button_list(["Apple", {"text": "Banana"}, {"text": "Pear"}])
        radio_grp_1_lay = QtWidgets.QHBoxLayout()
        radio_grp_1_lay.addWidget(radio_group_h)
        radio_grp_1_lay.addStretch()

        app_data = [
            MButtonGroupData(text="Maya", icon=MIcon("app-maya.png")),
            MButtonGroupData(text="Nuke", icon=MIcon("app-nuke.png")),
            MButtonGroupData(text="Houdini", icon=MIcon("app-houdini.png")),
        ]

        radio_group_v = MRadioButtonGroup(orientation=QtCore.Qt.Orientation.Vertical)
        radio_group_v.set_button_list(app_data)

        radio_group_button_h = MRadioButtonGroup()
        radio_group_button_h.set_button_list(app_data)
        radio_grp_h_lay = QtWidgets.QHBoxLayout()
        radio_grp_h_lay.addWidget(radio_group_button_h)
        radio_grp_h_lay.addStretch()

        radio_group_button_v = MRadioButtonGroup(
            orientation=QtCore.Qt.Orientation.Vertical
        )
        radio_group_button_v.set_button_list(app_data)

        self.register_field("value1", -1)
        self.register_field(
            "value1_text", functools.partial(self.value_to_text, "value1", app_data)
        )
        self.register_field("value2", 0)
        self.register_field(
            "value2_text", functools.partial(self.value_to_text, "value2", app_data)
        )
        self.register_field("value3", -1)
        self.register_field(
            "value3_text", functools.partial(self.value_to_text, "value3", app_data)
        )

        button1 = MPushButton(text="Group 1")
        button2 = MPushButton(text="Group 2")
        button3 = MPushButton(text="Group 3")
        button1.clicked.connect(functools.partial(self.slot_change_value, "value1"))
        button2.clicked.connect(functools.partial(self.slot_change_value, "value2"))
        button3.clicked.connect(functools.partial(self.slot_change_value, "value3"))

        self.bind("value1", radio_group_v, "dayu_checked", signal="checked_changed")
        self.bind(
            "value2", radio_group_button_h, "dayu_checked", signal="checked_changed"
        )
        self.bind(
            "value3", radio_group_button_v, "dayu_checked", signal="checked_changed"
        )
        self.bind("value1_text", button1, "text")
        self.bind("value2_text", button2, "text")
        self.bind("value3_text", button3, "text")

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("MRadioButtonGroup: orientation=Qt.Horizontal "))
        main_lay.addLayout(radio_grp_1_lay)
        main_lay.addWidget(MDivider("MRadioButtonGroup: orientation=Qt.Vertical"))
        main_lay.addWidget(radio_group_v)
        main_lay.addWidget(button1)
        main_lay.addWidget(
            MDivider("MRadioButtonGroup: orientation=Qt.Horizontal type=button")
        )
        main_lay.addLayout(radio_grp_h_lay)
        main_lay.addWidget(button2)
        main_lay.addWidget(
            MDivider("MRadioButtonGroup: orientation=Qt.Vertical, type=button")
        )
        main_lay.addWidget(radio_group_button_v)
        main_lay.addWidget(button3)
        main_lay.addStretch()
        self.setLayout(main_lay)

    def value_to_text(self, field: str, data_list: list[MButtonGroupData]) -> str:
        return (
            "Please Select One"
            if self.field(field) < 0
            else f"You Selected [{data_list[self.field(field)]['text']}]"
        )

    def slot_change_value(self, attr: str):
        import random

        self.set_field(attr, random.randrange(0, 3))


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = RadioButtonGroupExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtCore, QtWidgets

from qtui.dayu.collapse import MCollapse
from qtui.dayu.label import MLabel
from qtui.dayu.types import MCollapseData


class CollapseExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MCollapse")
        self._init_ui()

    def _init_ui(self):
        label_1 = MLabel(
            "Steve Jobs was an American inventor, entrepreneur, and co-founder of Apple Inc. He was born on February 24, 1955, in San Francisco, California."  # noqa: E501
        )
        label_2 = MLabel(
            "Stephen Gary Wozniak was an American computer engineer who co-founded Apple Inc. with Steve Jobs. He studied at the University of Colorado and then transferred to the University of California, Berkeley, where he earned a degree in electrical engineering and computer science in 1987."  # noqa: E501
        )
        label_3 = MLabel(
            "Jonathan Ive is an industrial designer who is currently the designer and senior vice president of Apple Inc. He is a British knight."  # noqa: E501
        )
        label_1.setWordWrap(True)
        label_2.setWordWrap(True)
        label_3.setWordWrap(True)
        section_list = [
            MCollapseData(title="Steve Jobs", expand=True, content=label_1),
            MCollapseData(
                title="This is a closable collapse item",
                expand=True,
                content=MLabel("This is a closable collapse item"),
                closable=True,
            ),
            MCollapseData(title="Stephen Gary Wozniak", expand=True, content=label_2),
        ]

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.setSpacing(1)
        for section in section_list:
            section_item = MCollapse(**section)
            section_item.setSizePolicy(
                QtWidgets.QSizePolicy.Policy.Minimum,
                QtWidgets.QSizePolicy.Policy.Minimum,
            )
            main_lay.addWidget(section_item)
        main_lay.addStretch()
        main_lay.setAlignment(QtCore.Qt.AlignmentFlag.AlignTop)
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = CollapseExample()
        MTheme().apply(test)
        test.show()



import contextlib
import signal
import sys
from pathlib import Path
from typing import Generic, TypeVar

from qtpy import QtCore, QtGui, QtWidgets
from qtpy.QtSvg import QSvgRenderer

T = TypeVar("T", bound=QtGui.QPixmap | QtGui.QIcon)


class MStaticCacheDict(Generic[T]):
    _render = QSvgRenderer()

    def __init__(self, cls: type[T]):
        super().__init__()
        self.cls = cls
        self._cache_pix_dict: dict[str, T] = {}

    def _render_svg(self, svg_path: str, replace_color: str = "") -> T:
        from ..theme import MTheme

        replace_color = replace_color or MTheme().icon_color

        data_content = Path(svg_path).read_text().replace("#555555", replace_color)
        self._render.load(QtCore.QByteArray(data_content.encode()))
        pix = QtGui.QPixmap(128, 128)
        pix.fill(QtCore.Qt.GlobalColor.transparent)
        painter = QtGui.QPainter(pix)
        self._render.render(painter)
        painter.end()
        return self.cls(pix)

    def __call__(self, name: str, color: str | None = "") -> T:
        from ..utils import get_static_file

        try:
            color = color or ""
            full_path = get_static_file(name).as_posix()
            key = f"{full_path.lower()}{color}"
            pix_map = self._cache_pix_dict.get(key)
            if pix_map is None:
                if full_path.endswith("svg"):
                    pix_map = self._render_svg(full_path, color)
                else:
                    pix_map = self.cls(full_path)
                self._cache_pix_dict[key] = pix_map
            return pix_map
        except FileNotFoundError:
            return self.cls()


def get_scale_factor() -> tuple[float, float]:
    """
    Get the scale factor for the x and y dimensions.

    Args:
        None
    Returns:
        tuple[float, float]: The scale factor for the x and y dimensions.
    """
    if not QtWidgets.QApplication.instance():
        QtWidgets.QApplication([])
    standard_dpi = 96.0
    scale_factor_x = (
        QtWidgets.QApplication.primaryScreen().logicalDotsPerInchX() / standard_dpi
    )
    scale_factor_y = (
        QtWidgets.QApplication.primaryScreen().logicalDotsPerInchY() / standard_dpi
    )
    return scale_factor_x, scale_factor_y


@contextlib.contextmanager
def application():
    app = QtWidgets.QApplication.instance()

    if not app:
        signal.signal(signal.SIGINT, signal.SIG_DFL)
        app = QtWidgets.QApplication(sys.argv)
        yield app
        app.exec()
    else:
        yield app


MPixmap = MStaticCacheDict(QtGui.QPixmap)
MIcon = MStaticCacheDict(QtGui.QIcon)



# Import third-party modules
from qtpy import QtCore, QtGui, QtWidgets

from .mixin import hover_shadow_mixin, property_mixin
from .utils import str_to_qbytearray


@hover_shadow_mixin
@property_mixin
class MPopup(QtWidgets.QFrame):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowFlags(QtCore.Qt.WindowType.Popup)
        self.mouse_pos: QtCore.QPoint | None = None
        self.setProperty("movable", True)
        self.setProperty("animatable", True)
        QtCore.QTimer.singleShot(0, self.post_init)

        self._opacity_anim = QtCore.QPropertyAnimation(
            self, str_to_qbytearray("windowOpacity")
        )
        self.setProperty("anim_opacity_duration", 300)
        self.setProperty("anim_opacity_curve", "OutCubic")
        self.setProperty("anim_opacity_start", 0)
        self.setProperty("anim_opacity_end", 1)

        self._size_anim = QtCore.QPropertyAnimation(self, str_to_qbytearray("size"))
        self.setProperty("anim_size_duration", 300)
        self.setProperty("anim_size_curve", "OutCubic")
        self.setProperty("border_radius", 15)

    def post_init(self) -> None:
        start_size = self.property("anim_size_start")
        size = self.sizeHint()
        start_size = start_size if start_size else QtCore.QSize(0, size.height())
        end_size = self.property("anim_size_end")
        end_size = end_size if end_size else size
        self.setProperty("anim_size_start", start_size)
        self.setProperty("anim_size_end", end_size)

    def update_mask(self):
        rect_path = QtGui.QPainterPath()
        end_size = self.property("anim_size_end")
        rect = QtCore.QRectF(0, 0, end_size.width(), end_size.height())
        radius = self.property("border_radius")
        rect_path.addRoundedRect(rect, radius, radius)
        self.setMask(QtGui.QRegion(rect_path.toFillPolygon().toPolygon()))

    def _get_curve(self, value: str) -> QtCore.QEasingCurve.Type:
        curve = getattr(QtCore.QEasingCurve, value, None)
        if not curve:
            raise TypeError("Invalid QEasingCurve")
        return curve

    def _set_border_radius(self, value: int) -> None:
        QtCore.QTimer.singleShot(0, self.update_mask)

    def _set_anim_opacity_duration(self, value: int) -> None:
        self._opacity_anim.setDuration(value)

    def _set_anim_opacity_curve(self, value: str) -> None:
        self._opacity_anim.setEasingCurve(self._get_curve(value))

    def _set_anim_opacity_start(self, value: float) -> None:
        self._opacity_anim.setStartValue(value)

    def _set_anim_opacity_end(self, value: float) -> None:
        self._opacity_anim.setEndValue(value)

    def _set_anim_size_duration(self, value: int) -> None:
        self._size_anim.setDuration(value)

    def _set_anim_size_curve(self, value: str) -> None:
        self._size_anim.setEasingCurve(self._get_curve(value))

    def _set_anim_size_start(self, value: QtCore.QSize) -> None:
        self._size_anim.setStartValue(value)

    def _set_anim_size_end(self, value: QtCore.QSize) -> None:
        self._size_anim.setEndValue(value)
        QtCore.QTimer.singleShot(0, self.update_mask)

    def start_anim(self) -> None:
        self._size_anim.start()
        self._opacity_anim.start()

    def mousePressEvent(self, event: QtGui.QMouseEvent) -> None:  # noqa: N802
        if event.button() == QtCore.Qt.MouseButton.LeftButton:
            self.mouse_pos = event.pos()
        return super().mousePressEvent(event)

    def mouseReleaseEvent(self, event: QtGui.QMouseEvent) -> None:  # noqa: N802
        self.mouse_pos = None
        return super().mouseReleaseEvent(event)

    def mouseMoveEvent(self, event: QtGui.QMouseEvent) -> None:  # noqa: N802
        if (
            event.buttons() == QtCore.Qt.MouseButton.LeftButton
            and self.mouse_pos
            and self.property("movable")
        ):
            self.move(self.mapToGlobal(event.pos() - self.mouse_pos))
        return super().mouseMoveEvent(event)

    def show(self) -> None:
        if self.property("animatable"):
            self.start_anim()
        self.move(QtGui.QCursor.pos())
        super().show()
        self.activateWindow()



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MIcon
from qtui.dayu.radio_button import MRadioButton


class RadioButtonExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Example for MRadioButton")
        widget_1 = QtWidgets.QWidget()
        lay_1 = QtWidgets.QHBoxLayout()
        lay_1.addWidget(MRadioButton("Maya"))
        lay_1.addWidget(MRadioButton("Nuke"))
        lay_1.addWidget(MRadioButton("Houdini"))
        widget_1.setLayout(lay_1)

        check_box_icon_1 = MRadioButton("Folder")
        check_box_icon_1.setIcon(MIcon("folder_fill.svg"))
        check_box_icon_2 = MRadioButton("Media")
        check_box_icon_2.setIcon(MIcon("media_fill.svg"))
        check_box_icon_3 = MRadioButton("User")
        check_box_icon_3.setIcon(MIcon("user_fill.svg"))
        check_box_icon_2.setChecked(True)
        widget_2 = QtWidgets.QWidget()
        lay_2 = QtWidgets.QHBoxLayout()
        lay_2.addWidget(check_box_icon_1)
        lay_2.addWidget(check_box_icon_2)
        lay_2.addWidget(check_box_icon_3)
        widget_2.setLayout(lay_2)

        check_box_single = MRadioButton("Alipay")
        check_box_single.setChecked(True)
        check_box_single.setEnabled(False)

        check_box_bind = MRadioButton("Data Bind")
        label = MLabel()
        button = MPushButton(text="Change State")
        button.clicked.connect(
            lambda: self.set_field("checked", not self.field("checked"))
        )
        self.register_field("checked", True)
        self.register_field(
            "checked_text", lambda: "Yes!" if self.field("checked") else "No!!"
        )
        self.bind("checked", check_box_bind, "checked", signal="toggled")
        self.bind("checked_text", label, "text")

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Basic"))
        main_lay.addWidget(widget_1)
        main_lay.addWidget(check_box_single)
        main_lay.addWidget(MDivider("Icon"))
        main_lay.addWidget(widget_2)
        main_lay.addWidget(MDivider("Data Bind"))
        main_lay.addWidget(check_box_bind)
        main_lay.addWidget(label)
        main_lay.addWidget(button)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = RadioButtonExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtCore, QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.label import MLabel
from qtui.dayu.message import MMessage
from qtui.dayu.tab_widget import MTabWidget


class TabWidgetExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()
        self.resize(500, 500)

    def _init_ui(self):
        main_lay = QtWidgets.QVBoxLayout()

        tab_card = MTabWidget()
        tab_card.addTab(MLabel("test 1"), "Current Element")
        tab_card.addTab(MLabel("test 2"), "Linked Assets")
        tab_card.addTab(MLabel("test 2"), "Hero Shots")
        tab_card.addTab(MLabel("test 3"), "Linked Metadata")

        self.tab_closable = MTabWidget()
        self.tab_closable.setTabsClosable(True)
        self.tab_closable.addTab(MLabel("test 1"), "Label One")
        self.tab_closable.addTab(MLabel("test 2"), "Label Two")
        self.tab_closable.addTab(MLabel("test 3"), "Label Three")
        self.tab_closable.tabCloseRequested.connect(self.slot_close_tab)
        main_lay.addWidget(MDivider("Normal"))
        main_lay.addWidget(tab_card)
        main_lay.addWidget(MDivider("Closable"))
        main_lay.addWidget(self.tab_closable)
        self.setLayout(main_lay)

    @QtCore.Slot(int)
    def slot_close_tab(self, index: int):
        if index > 0:
            text = self.tab_closable.tabText(index)
            self.tab_closable.removeTab(index)
            MMessage.info(
                f"Successfully closed a label: {text}", closable=True, parent=self
            )
        else:
            MMessage.warning(
                "Please do not close the first label", closable=True, parent=self
            )


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = TabWidgetExample()
        MTheme().apply(test)
        test.show()



from __future__ import annotations

import contextlib
import re
from collections.abc import Callable, Iterator, Sequence
from functools import partial
from typing import Any, cast

from qtpy import QtCore, QtGui, QtWidgets

from . import utils
from .line_edit import MLineEdit
from .mixin import property_mixin
from .popup import MPopup
from .types import MenuItemData


@property_mixin
class ScrollableMenuBase(QtWidgets.QMenu):
    delta_y: int = 0
    dirty: bool = True
    ignore_auto_scroll: bool = False

    def __init__(self, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self._maximum_height = self.maximumHeight()
        self._action_rects: list[QtCore.QRect] = []

        self.scroll_timer = QtCore.QTimer(self, interval=50, singleShot=True)
        self.scroll_timer.timeout.connect(self.check_scroll)
        self.scroll_timer.setProperty("defaultInterval", 50)
        self.delay_timer = QtCore.QTimer(self, interval=100, singleShot=True)

        self.set_max_item_count(0)

    def _set_max_scroll_count(self, value: float):
        self.set_max_item_count(int(value * 2.2))

    @property
    def action_rects(self):
        if self.dirty or not self._action_rects:
            del self._action_rects[:]
            offset = self.offset()
            for action in self.actions():
                geo = super().actionGeometry(action)
                if offset:
                    geo.moveTop(geo.y() - offset)
                self._action_rects.append(geo)
            self.dirty = False
        return self._action_rects

    def iter_action_rects(self) -> Iterator[tuple[QtGui.QAction, QtCore.QRect]]:
        yield from zip(self.actions(), self.action_rects, strict=False)

    def set_max_item_count(self, count: int):
        style = self.style()
        opt = QtWidgets.QStyleOptionMenuItem()
        opt.initFrom(self)

        a = QtGui.QAction("fake action", self)
        self.initStyleOption(opt, a)
        size = QtCore.QSize()
        fm = self.fontMetrics()
        size.setWidth(
            fm.boundingRect(
                QtCore.QRect(), QtCore.Qt.TextFlag.TextSingleLine, a.text()
            ).width()
        )
        size.setHeight(fm.height())
        self.default_item_height = style.sizeFromContents(
            QtWidgets.QStyle.ContentsType.CT_MenuItem, opt, size, self
        ).height()

        if not count:
            self.setMaximumHeight(self._maximum_height)
        else:
            fw = style.pixelMetric(style.PixelMetric.PM_MenuHMargin, None, self)
            vmargin = style.pixelMetric(style.PixelMetric.PM_MenuHMargin, opt, self)
            scroll_height = self.scroll_height(style)
            self.setMaximumHeight(
                self.default_item_height * count + (fw + vmargin + scroll_height) * 2
            )
        self.dirty = True

    def scroll_height(self, style: QtWidgets.QStyle) -> int:
        return (
            style.pixelMetric(style.PixelMetric.PM_MenuScrollerHeight, None, self) * 2
        )

    def is_scrollable(self) -> bool:
        return (
            self.property("scrollable") and self.height() < super().sizeHint().height()
        )

    def check_scroll(self):
        pos = self.mapFromGlobal(QtGui.QCursor.pos())
        delta = max(2, int(self.default_item_height * 0.25))
        if self.scroll_up_rect.contains(pos):
            delta *= -1
        elif not self.scroll_down_rect.contains(pos):
            return
        if self.scroll_by(delta):
            self.scroll_timer.start(self.scroll_timer.property("defaultInterval"))

    def offset(self) -> int:
        if self.is_scrollable():
            return self.delta_y - self.scroll_height(self.style())
        return 0

    def translated_action_geometry(self, action: QtGui.QAction) -> QtCore.QRect:
        return self.action_rects[self.actions().index(action)]

    def ensure_visible(self, action: QtGui.QAction):
        style = self.style()
        fw = style.pixelMetric(style.PixelMetric.PM_MenuPanelWidth, None, self)
        hmargin = style.pixelMetric(style.PixelMetric.PM_MenuHMargin, None, self)
        vmargin = style.pixelMetric(style.PixelMetric.PM_MenuVMargin, None, self)
        scroll_height = self.scroll_height(style)
        extent = fw + hmargin + vmargin + scroll_height
        r = self.rect().adjusted(0, extent, 0, -extent)
        geo = self.translated_action_geometry(action)
        if geo.top() < r.top():
            self.scroll_by(-(r.top() - geo.top()))
        elif geo.bottom() > r.bottom():
            self.scroll_by(geo.bottom() - r.bottom())

    def scroll_by(self, step: int) -> bool:
        if step < 0:
            new_delta = max(0, self.delta_y + step)
            if new_delta == self.delta_y:
                return False
        elif step > 0:
            new_delta = self.delta_y + step
            style = self.style()
            scroll_height = self.scroll_height(style)
            bottom = self.height() - scroll_height

            for last_action in reversed(self.actions()):
                if last_action.isVisible():
                    last_bottom = (
                        self.actionGeometry(last_action).bottom()
                        - new_delta
                        + scroll_height
                    )
                    if last_bottom < bottom:
                        new_delta -= bottom - last_bottom
                    if new_delta == self.delta_y:
                        return False
                    break
        else:
            return False

        self.delta_y = new_delta
        self.dirty = True
        self.update()
        return True

    # class methods reimplementation

    def actionAt(self, pos: QtCore.QPoint) -> QtGui.QAction | None:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        for action, rect in self.iter_action_rects():
            if rect.contains(pos):
                return action

    def sizeHint(self) -> QtCore.QSize:  # noqa: N802
        hint = super().sizeHint()
        if hint.height() > self.maximumHeight():
            hint.setHeight(self.maximumHeight())
        return hint

    def eventFilter(self, source: QtWidgets.QObject, event: QtCore.QEvent) -> bool:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        if event.type() == QtCore.QEvent.Type.Show:
            if self.is_scrollable() and self.delta_y:
                action = source.menuAction()
                self.ensure_visible(action)
                rect = self.translated_action_geometry(action)
                delta = rect.topLeft() - self.actionGeometry(action).topLeft()
                source.move(source.pos() + delta)
            return False
        return super().eventFilter(source, event)

    def event(self, event: QtCore.QEvent) -> bool:
        if not self.is_scrollable():
            return super().event(event)
        if event.type() == QtCore.QEvent.Type.KeyPress:
            event = cast(QtGui.QKeyEvent, event)
            if event.key() in (
                QtCore.Qt.Key.Key_Up,
                QtCore.Qt.Key.Key_Down,
            ):
                res = super().event(event)
                action = self.activeAction()
                if action:
                    self.ensure_visible(action)
                    self.update()
                return res
        elif event.type() in (
            QtCore.QEvent.Type.MouseButtonPress,
            QtCore.QEvent.Type.MouseButtonDblClick,
        ):
            event = cast(QtGui.QMouseEvent, event)
            pos = event.pos()
            if self.scroll_up_rect.contains(pos) or self.scroll_down_rect.contains(pos):
                if event.button() == QtCore.Qt.MouseButton.LeftButton:
                    step = max(2, int(self.default_item_height * 0.25))
                    if self.scroll_up_rect.contains(pos):
                        step *= -1
                    self.scroll_by(step)
                    self.scroll_timer.start(200)
                    self.ignore_auto_scroll = True
                return True
        elif event.type() == QtCore.QEvent.Type.MouseButtonRelease:
            event = cast(QtGui.QMouseEvent, event)
            pos = event.pos()
            self.scroll_timer.stop()
            if not (
                self.scroll_up_rect.contains(pos) or self.scroll_down_rect.contains(pos)
            ):
                action = self.actionAt(pos)
                if action:
                    action.trigger()
                    self.close()
            return True
        return super().event(event)

    def timerEvent(self, event: QtCore.QTimerEvent):  # noqa: N802
        if not self.is_scrollable():
            # ignore internal timer event for reopening popups
            super().timerEvent(event)

    def mouseMoveEvent(self, event: QtGui.QMouseEvent):  # noqa: N802
        if not self.is_scrollable():
            super().mouseMoveEvent(event)
            return

        pos = event.pos()
        if (
            pos.y() < self.scroll_up_rect.bottom()
            or pos.y() > self.scroll_down_rect.top()
        ):
            if not self.ignore_auto_scroll and not self.scroll_timer.isActive():
                self.scroll_timer.start(200)
            return
        self.ignore_auto_scroll = False

        old_action = self.activeAction()
        if not self.rect().contains(pos):
            action = None
        else:
            y = event.y()
            for action, rect in self.iter_action_rects():
                if rect.y() <= y <= rect.y() + rect.height():
                    action = action
                    break
            else:
                action = None

        self.setActiveAction(action) if action else None
        if action and not action.isSeparator():

            def ensureVisible():  # noqa: N802
                self.delay_timer.timeout.disconnect()
                self.ensure_visible(action)

            with contextlib.suppress(Exception):
                self.delay_timer.disconnect()  # pyright: ignore[reportCallIssue]
            self.delay_timer.timeout.connect(ensureVisible)
            self.delay_timer.start(150)
        elif old_action and old_action.menu() and old_action.menu().isVisible():  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]

            def closeMenu():  # noqa: N802
                self.delay_timer.timeout.disconnect()
                old_action.menu().hide()  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]

            self.delay_timer.timeout.connect(closeMenu)
            self.delay_timer.start(50)
        self.update()

    def wheelEvent(self, event: QtGui.QWheelEvent):  # noqa: N802
        if not self.is_scrollable():
            return
        self.delay_timer.stop()
        if event.angleDelta().y() < 0:
            self.scroll_by(self.default_item_height)
        else:
            self.scroll_by(-self.default_item_height)

    def showEvent(self, event: QtGui.QShowEvent):  # noqa: N802
        if self.is_scrollable():
            self.delta_y = 0
            self.dirty = True
            for action in self.actions():
                if action.menu():
                    action.menu().installEventFilter(self)
            self.ignore_auto_scroll = False
        super().showEvent(event)

    def hideEvent(self, event: QtGui.QHideEvent):  # noqa: N802
        for action in self.actions():
            if action.menu():
                action.menu().removeEventFilter(self)
        super().hideEvent(event)

    def resizeEvent(self, event: QtGui.QResizeEvent):  # noqa: N802
        super().resizeEvent(event)

        style = self.style()
        margins = self.contentsMargins()
        l, t, r, b = margins.left(), margins.top(), margins.right(), margins.bottom()  # noqa: E741
        fw = style.pixelMetric(style.PixelMetric.PM_MenuPanelWidth, None, self)
        hmargin = style.pixelMetric(style.PixelMetric.PM_MenuHMargin, None, self)
        vmargin = style.pixelMetric(style.PixelMetric.PM_MenuVMargin, None, self)
        left_margin = fw + hmargin + l
        top_margin = fw + vmargin + t
        bottom_margin = fw + vmargin + b
        content_width = self.width() - (fw + hmargin) * 2 - l - r

        scroll_height = self.scroll_height(style)
        self.scroll_up_rect = QtCore.QRect(
            left_margin, top_margin, content_width, scroll_height
        )
        self.scroll_down_rect = QtCore.QRect(
            left_margin,
            self.height() - scroll_height - bottom_margin,
            content_width,
            scroll_height,
        )

    def paintEvent(self, event: QtGui.QPaintEvent):  # noqa: N802
        if not self.is_scrollable():
            super().paintEvent(event)
            return

        style = self.style()
        qp = QtGui.QPainter(self)
        rect = self.rect()
        empty_area = QtGui.QRegion(rect)

        menu_opt = QtWidgets.QStyleOptionMenuItem()
        menu_opt.initFrom(self)
        menu_opt.state = style.StateFlag.State_None  # pyright: ignore[reportAttributeAccessIssue]
        menu_opt.maxIconWidth = 0  # pyright: ignore[reportAttributeAccessIssue]
        menu_opt.tabWidth = 0  # pyright: ignore[reportAttributeAccessIssue]
        style.drawPrimitive(style.PrimitiveElement.PE_PanelMenu, menu_opt, qp, self)

        fw = style.pixelMetric(style.PixelMetric.PM_MenuPanelWidth, None, self)
        top_edge = self.scroll_up_rect.bottom()
        bottom_edge = self.scroll_down_rect.top()
        offset = self.offset()
        qp.save()
        qp.translate(0, -offset)
        # offset translation is required in order to allow correct fade animations
        action_rect = next(self.iter_action_rects())[1]
        for action, action_rect in self.iter_action_rects():
            action_rect = self.translated_action_geometry(action)
            if action_rect.bottom() < top_edge:
                continue
            if action_rect.top() > bottom_edge:
                continue

            visible = QtCore.QRect(action_rect)
            if action_rect.bottom() > bottom_edge:
                visible.setBottom(bottom_edge)
            elif action_rect.top() < top_edge:
                visible.setTop(top_edge)
            visible = QtGui.QRegion(visible.translated(0, offset))
            qp.setClipRegion(visible)
            empty_area -= visible.translated(0, -offset)

            opt = QtWidgets.QStyleOptionMenuItem()
            self.initStyleOption(opt, action)
            opt.rect = action_rect.translated(0, offset)  # pyright: ignore[reportAttributeAccessIssue]
            style.drawControl(style.ControlElement.CE_MenuItem, opt, qp, self)
        qp.restore()

        cursor = self.mapFromGlobal(QtGui.QCursor.pos())
        up_data = (False, self.delta_y > 0, self.scroll_up_rect)
        down_data = (
            True,
            action_rect.bottom() - 2 > bottom_edge,
            self.scroll_down_rect,
        )

        for is_down, enabled, scroll_rect in up_data, down_data:
            qp.setClipRect(scroll_rect)

            scroll_opt = QtWidgets.QStyleOptionMenuItem()
            scroll_opt.initFrom(self)
            scroll_opt.state = style.StateFlag.State_None  # pyright: ignore[reportAttributeAccessIssue]
            scroll_opt.state |= (  # pyright: ignore[reportAttributeAccessIssue]
                style.StateFlag.State_DownArrow
                if is_down
                else style.StateFlag.State_UpArrow
            )
            scroll_opt.checkType = scroll_opt.CheckType.NotCheckable  # pyright: ignore[reportAttributeAccessIssue]
            scroll_opt.maxIconWidth = scroll_opt.tabWidth = 0  # pyright: ignore[reportAttributeAccessIssue]
            scroll_opt.rect = scroll_rect  # pyright: ignore[reportAttributeAccessIssue]
            if enabled:
                if scroll_rect.contains(cursor):
                    frame = QtWidgets.QStyleOptionMenuItem()
                    frame.initFrom(self)
                    frame.rect = scroll_rect  # pyright: ignore[reportAttributeAccessIssue]
                    frame.state |= (  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
                        style.StateFlag.State_Selected | style.StateFlag.State_Enabled
                    )
                    style.drawControl(style.ControlElement.CE_MenuItem, frame, qp, self)

                scroll_opt.state |= style.StateFlag.State_Enabled  # pyright: ignore[reportAttributeAccessIssue]
                scroll_opt.palette.setCurrentColorGroup(  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
                    QtGui.QPalette.ColorGroup.Active
                )
            else:
                scroll_opt.palette.setCurrentColorGroup(  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
                    QtGui.QPalette.ColorGroup.Disabled
                )
            style.drawControl(
                style.ControlElement.CE_MenuScroller, scroll_opt, qp, self
            )

        if fw:
            border_reg = QtGui.QRegion()
            border_reg |= QtGui.QRegion(QtCore.QRect(0, 0, fw, self.height()))
            border_reg |= QtGui.QRegion(
                QtCore.QRect(self.width() - fw, 0, fw, self.height())
            )
            border_reg |= QtGui.QRegion(QtCore.QRect(0, 0, self.width(), fw))
            border_reg |= QtGui.QRegion(
                QtCore.QRect(0, self.height() - fw, self.width(), fw)
            )
            qp.setClipRegion(border_reg)
            empty_area -= border_reg
            frame = QtWidgets.QStyleOptionFrame()
            frame.rect = rect  # pyright: ignore[reportAttributeAccessIssue]
            frame.palette = self.palette()  # pyright: ignore[reportAttributeAccessIssue]
            frame.state = style.StateFlag.State_None  # pyright: ignore[reportAttributeAccessIssue]
            frame.lineWidth = style.pixelMetric(style.PixelMetric.PM_MenuPanelWidth)  # pyright: ignore[reportAttributeAccessIssue]
            frame.midLineWidth = 0  # pyright: ignore[reportAttributeAccessIssue]
            style.drawPrimitive(style.PrimitiveElement.PE_FrameMenu, frame, qp, self)

        qp.setClipRegion(empty_area)
        menu_opt.state = style.StateFlag.State_None  # pyright: ignore[reportAttributeAccessIssue]
        menu_opt.rect = menu_opt.menuRect = rect  # pyright: ignore[reportAttributeAccessIssue]
        style.drawControl(style.ControlElement.CE_MenuEmptyArea, menu_opt, qp, self)


@property_mixin
class SearchableMenuBase(ScrollableMenuBase):
    def __init__(self, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self.search_popup = MPopup(self)
        self.search_popup.setVisible(False)
        self.search_bar = MLineEdit(parent=self)
        self.search_label = QtWidgets.QLabel()

        self.search_bar.textChanged.connect(self.slot_search_change)
        self.search_bar.keyPressEvent = partial(
            self.search_key_event, self.search_bar.keyPressEvent
        )
        self.aboutToHide.connect(lambda: self.search_bar.setText(""))

        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.search_label)
        layout.addWidget(self.search_bar)
        self.search_popup.setLayout(layout)

        self.setProperty("search_placeholder", self.tr("Search Action..."))
        self.setProperty("search_label", self.tr("Search Action..."))

        self.setProperty("searchable", True)
        self.setProperty("search_re", "I")

    def search_key_event(
        self, call: Callable[[QtGui.QKeyEvent], Any], event: QtGui.QKeyEvent
    ):
        key = event.key()
        # NOTES: support menu original key event on search bar
        if key in (
            QtCore.Qt.Key.Key_Up,
            QtCore.Qt.Key.Key_Down,
            QtCore.Qt.Key.Key_Return,
            QtCore.Qt.Key.Key_Enter,
        ):
            super().keyPressEvent(event)
        elif key == QtCore.Qt.Key.Key_Tab:
            self.search_bar.setFocus()
        return call(event)

    def _set_search_label(self, value: str):
        self.search_label.setText(value)

    def _set_search_placeholder(self, value: str):
        self.search_bar.setPlaceholderText(value)

    def _set_search_re(self, value: str): ...

    def slot_search_change(self, text: str):
        flags = 0
        for m in self.property("search_re") or "":
            flags |= getattr(re, m.upper(), 0)
        search_reg = re.compile(rf".*{text}.*", flags)
        self._update_search(search_reg)

    def _update_search(
        self, search_reg: str | re.Pattern[str], parent_menu: MMenu | None = None
    ):
        actions = parent_menu.actions() if parent_menu else self.actions()
        vis_list: list[QtGui.QAction] = []
        for action in actions:
            menu = cast(MMenu, action.menu())
            if not menu:
                is_match = bool(re.match(search_reg, action.text()))
                action.setVisible(is_match)
                if is_match:
                    vis_list.append(action)
            else:
                is_match = bool(re.match(search_reg, menu.title()))
                self._update_search("" if is_match else search_reg, menu)

        if parent_menu:
            parent_menu.menuAction().setVisible(bool(vis_list) or not search_reg)

    def keyPressEvent(self, event: QtGui.QKeyEvent):  # noqa: N802
        key = event.key()
        if self.property("searchable"):
            # NOTES(timmyliang): 26 character trigger search bar
            if 65 <= key <= 90:
                char = chr(key)
                self.search_bar.setText(char)
                self.search_bar.setFocus()
                self.search_bar.selectAll()
                width = self.sizeHint().width()
                width = width if width >= 50 else 50
                offset = QtCore.QPoint(width, 0)
                self.search_popup.move(self.pos() + offset)
                self.search_popup.show()
            elif key == QtCore.Qt.Key.Key_Escape:
                self.search_bar.setText("")
                self.search_popup.hide()
        return super().keyPressEvent(event)


@property_mixin
class MMenu(ScrollableMenuBase):
    value_changed = QtCore.Signal(object)

    def __init__(
        self,
        exclusive: bool = True,
        cascader: bool = False,
        title: str = "",
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(title=title, parent=parent)
        self.setProperty("cascader", cascader)
        self.setCursor(QtCore.Qt.CursorShape.PointingHandCursor)
        self._action_group = QtGui.QActionGroup(self)
        self._action_group.setExclusive(exclusive)
        self._action_group.triggered.connect(self.on_action_triggered)
        self._load_data_func: (
            Callable[[], Sequence[str | int | float | MenuItemData]] | None
        ) = None
        self.set_value("")
        self.set_data([])
        self.set_separator("/")

    def set_separator(self, chr: str):
        self.setProperty("separator", chr)

    def set_load_callback(
        self, func: Callable[[], Sequence[str | int | float | MenuItemData]]
    ):
        """Set the load data callback function"""
        self._load_data_func = func
        self.aboutToShow.connect(self.on_fetch_data)

    def on_fetch_data(self):
        if self._load_data_func:
            data_list = self._load_data_func()
            self.set_data(data_list)

    def set_value(self, data: list[Any] | str | int | float):
        assert isinstance(data, list | str | int | float)
        if self.property("cascader") and isinstance(data, str):
            data = data.split(self.property("separator"))
        self.setProperty("value", data)

    def _set_value(self, value: Any):
        data_list: list[Any] = value if isinstance(value, list) else [value]  # pyright: ignore[reportUnknownVariableType]
        flag = False
        for act in self._action_group.actions():
            checked = act.property("value") in data_list
            if act.isChecked() != checked:
                act.setChecked(checked)
                flag = True
        if flag:
            self.value_changed.emit(value)

    def _add_menu(self, parent_menu: MMenu, data_dict: utils.MenuItemData):
        if "children" in data_dict:
            menu = MMenu(title=data_dict.get("label", ""), parent=self)
            menu.setProperty("value", data_dict.get("value"))
            parent_menu.addMenu(menu)
            if parent_menu is not self:
                menu.setProperty("parent_menu", parent_menu)
            for i in data_dict.get("children", []):
                self._add_menu(menu, i)
        else:
            action = self._action_group.addAction(
                utils.display_formatter(data_dict.get("label"))
            )
            action.setProperty("value", data_dict.get("value"))
            action.setCheckable(True)
            action.setProperty("parent_menu", parent_menu)
            parent_menu.addAction(action)

    def set_data(self, option_list: Sequence[str | int | float | utils.MenuItemData]):
        assert isinstance(option_list, list)
        if option_list:
            if all(isinstance(i, str) for i in option_list):
                option_list = cast(list[str], option_list)
                option_list = utils.from_list_to_nested_dict(
                    option_list,
                    sep=self.property("separator"),
                )
            if all(isinstance(i, int | float) for i in option_list):
                option_list = [
                    utils.MenuItemData(value=i, label=str(i)) for i in option_list
                ]
        self.setProperty("data", option_list)

    def _set_data(self, option_list: list[utils.MenuItemData]):
        self.clear()
        for act in self._action_group.actions():
            self._action_group.removeAction(act)

        for data_dict in option_list:
            self._add_menu(self, data_dict)

    def _get_parent(self, result: list[Any], obj: QtGui.QAction):
        if obj.property("parent_menu"):
            parent_menu = obj.property("parent_menu")
            result.insert(0, parent_menu.property("value"))
            self._get_parent(result, parent_menu)

    def on_action_triggered(self, action: QtGui.QAction):
        current_data = action.property("value")
        if self.property("cascader"):
            selected_data = [current_data]
            self._get_parent(selected_data, action)
        else:
            if self._action_group.isExclusive():
                selected_data = current_data
            else:
                selected_data = [
                    act.property("value")
                    for act in self._action_group.actions()
                    if act.isChecked()
                ]
        self.set_value(selected_data)
        self.value_changed.emit(selected_data)

    def set_loader(self, func: Callable[..., Any]):
        self._load_data_func = func



"""
MRadioButton
"""

from qtpy import QtWidgets

from .mixin import cursor_mixin


@cursor_mixin
class MRadioButton(QtWidgets.QRadioButton):
    """
    MRadioButton just use stylesheet and set cursor shape when hover. No more extend.
    """

    def __init__(self, text: str = "", parent: QtWidgets.QWidget | None = None):
        super().__init__(text, parent=parent)



import functools

from qtpy import QtCore, QtWidgets

from qtui.dayu.combo_box import MComboBox
from qtui.dayu.divider import MDivider
from qtui.dayu.label import MLabel
from qtui.dayu.line_edit import MLineEdit
from qtui.dayu.menu import MMenu
from qtui.dayu.message import MMessage
from qtui.dayu.push_button import MPushButton
from qtui.dayu.tool_button import MToolButton


class LineEditExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MLineEdit")
        self._init_ui()

    def _init_ui(self):
        size_lay = QtWidgets.QHBoxLayout()
        line_edit_l = MLineEdit().large()
        line_edit_l.setPlaceholderText("large size")
        line_edit_m = MLineEdit().medium()
        line_edit_m.setPlaceholderText("default size")
        line_edit_s = MLineEdit().small()
        line_edit_s.setPlaceholderText("small size")
        size_lay.addWidget(line_edit_l)
        size_lay.addWidget(line_edit_m)
        size_lay.addWidget(line_edit_s)

        line_edit_tool_button = MLineEdit(text="MToolButton")
        line_edit_tool_button.set_prefix_widget(
            MToolButton().svg("user_line.svg").icon_only()
        )

        line_edit_label = MLineEdit(text="MLabel")
        tool_button = MLabel(text="User").mark().secondary()
        tool_button.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        tool_button.setFixedWidth(80)
        line_edit_label.set_prefix_widget(tool_button)

        line_edit_push_button = MLineEdit(text="MPushButton")
        push_button = MPushButton(text="Go").primary()
        push_button.setFixedWidth(40)
        line_edit_push_button.set_suffix_widget(push_button)

        search_engine_line_edit = MLineEdit().search_with_button().large()
        search_engine_line_edit.returnPressed.connect(self.slot_search)

        line_edit_options = MLineEdit()
        combobox = MComboBox()
        option_menu = MMenu()
        option_menu.set_separator("|")
        option_menu.set_data([r"http://", r"https://"])
        combobox.set_menu(option_menu)
        combobox.set_value("http://")
        combobox.setFixedWidth(100)
        line_edit_options.set_prefix_widget(combobox)

        delay_line_editor = MLineEdit()
        delay_display_label = MLabel()
        delay_button = MPushButton("Click to Edit Text")
        delay_line_editor.delay_text_changed.connect(delay_display_label.setText)
        delay_button.clicked.connect(
            functools.partial(delay_line_editor.setText, "Edited from code")
        )

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different size"))
        main_lay.addLayout(size_lay)
        main_lay.addWidget(MDivider("custom prefix and suffix widget"))
        main_lay.addWidget(line_edit_tool_button)
        main_lay.addWidget(line_edit_label)
        main_lay.addWidget(line_edit_push_button)
        main_lay.addWidget(MDivider("preset"))

        main_lay.addWidget(MLabel("error"))
        main_lay.addWidget(
            MLineEdit(text="waring: file d:/ddd/ccc.jpg not exists.").error()
        )
        main_lay.addWidget(MLabel("search"))
        main_lay.addWidget(MLineEdit().search_with_close().small())
        main_lay.addWidget(MLabel("search_engine"))
        main_lay.addWidget(search_engine_line_edit)
        main_lay.addWidget(MLabel("file"))
        main_lay.addWidget(MLineEdit().file().small())
        main_lay.addWidget(MLabel("folder"))
        main_lay.addWidget(MLineEdit().folder().small())
        main_lay.addWidget(MLabel("MLineEdit.options()"))
        main_lay.addWidget(line_edit_options)
        main_lay.addWidget(MDivider("Test delay Signal"))
        main_lay.addWidget(delay_line_editor)
        main_lay.addWidget(delay_display_label)
        main_lay.addWidget(delay_button)
        main_lay.addStretch()
        self.setLayout(main_lay)

    @QtCore.Slot()
    def slot_search(self):
        MMessage.info("查无此人", parent=self)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = LineEditExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtCore, QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.label import MLabel
from qtui.dayu.line_tab_widget import MLineTabWidget
from qtui.dayu.theme import MTheme
from qtui.dayu.tool_button import MToolButton


class LineTabWidgetExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Example for MLineTabWidget")
        self._init_ui()

    def _init_ui(self):
        main_lay = QtWidgets.QVBoxLayout()

        tab_center = MLineTabWidget()
        tab_center.add_tab(
            MLabel("test 1 " * 10), {"text": "Tab 1", "svg": "user_line.svg"}
        )
        tab_center.add_tab(MLabel("test 2 " * 10), {"svg": "calendar_line.svg"})
        tab_center.add_tab(MLabel("test 3 " * 10), "Tab 3")
        tab_center.tool_button_group.set_dayu_checked(0)

        tab_left = MLineTabWidget(alignment=QtCore.Qt.AlignmentFlag.AlignLeft)
        tab_left.add_tab(MLabel("test 1 " * 10), "Tab 1")
        tab_left.add_tab(MLabel("test 2 " * 10), "Tab 2")
        tab_left.add_tab(MLabel("test 3 " * 10), "Tab 3")
        tab_left.tool_button_group.set_dayu_checked(0)

        tab_right = MLineTabWidget(alignment=QtCore.Qt.AlignmentFlag.AlignRight)
        tab_right.add_tab(MLabel("test 1 " * 10), "Tab 1")
        tab_right.add_tab(MLabel("test 2 " * 10), "Tab 2")
        tab_right.add_tab(MLabel("test 3 " * 10), "Tab 3")
        tab_right.tool_button_group.set_dayu_checked(0)

        tab_huge = MLineTabWidget()
        tab_huge.set_dayu_size(MTheme().huge)
        tab_huge.add_tab(
            MLabel("test 1 " * 10), {"text": "Tab 1", "svg": "user_line.svg"}
        )
        tab_huge.add_tab(MLabel("test 2 " * 10), "Tab 2")
        tab_huge.add_tab(MLabel("test 3 " * 10), "Tab 3")
        tab_huge.tool_button_group.set_dayu_checked(0)

        tab_append_insert = MLineTabWidget()
        tab_append_insert.add_tab(
            MLabel("test 1 " * 10), {"text": "Tab 1", "svg": "user_line.svg"}
        )
        tab_append_insert.add_tab(MLabel("test 2 " * 10), "Tab 2")
        tab_append_insert.add_tab(MLabel("test 3 " * 10), "Tab 3")
        tab_append_insert.tool_button_group.set_dayu_checked(0)
        tab_append_insert.insert_widget(MLabel("Insert Label").h4())
        tab_append_insert.append_widget(
            MToolButton().svg("refresh_line.svg").icon_only()
        )

        main_lay.addWidget(MDivider("Center"))
        main_lay.addWidget(tab_center)
        main_lay.addSpacing(20)
        main_lay.addWidget(MDivider("Left"))
        main_lay.addWidget(tab_left)
        main_lay.addSpacing(20)
        main_lay.addWidget(MDivider("Right"))
        main_lay.addWidget(tab_right)
        main_lay.addSpacing(20)
        main_lay.addWidget(MDivider("Huge"))
        main_lay.addWidget(tab_huge)
        main_lay.addWidget(MDivider("append_widget/insert_widget"))
        main_lay.addWidget(tab_append_insert)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = LineTabWidgetExample()
        MTheme().apply(test)
        test.show()



"""MPage"""

import functools
from typing import Any

from qtpy import QtCore, QtWidgets

from . import utils
from .combo_box import MComboBox
from .field_mixin import MFieldMixin
from .label import MLabel
from .menu import MMenu
from .spin_box import MSpinBox
from .theme import MTheme
from .tool_button import MToolButton


class MPage(QtWidgets.QWidget, MFieldMixin):
    """
    MPage
    A long list can be divided into several pages by MPage,
    and only one page will be loaded at a time.
    """

    sig_page_changed = QtCore.Signal(int, int)

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.register_field("page_size_selected", 25)
        self.register_field(
            "page_size_list",
            [
                {"label": "25 - Fastest", "value": 25},
                {"label": "50 - Fast", "value": 50},
                {"label": "75 - Medium", "value": 75},
                {"label": "100 - Slow", "value": 100},
            ],
        )
        self.register_field("total", 0)
        self.register_field("current_page", 0)
        self.register_field(
            "total_page",
            lambda: utils.get_total_page(
                self.field("total"), self.field("page_size_selected")
            ),
        )
        self.register_field("total_page_text", lambda: str(self.field("total_page")))
        self.register_field(
            "display_text",
            lambda: utils.get_page_display_string(
                self.field("current_page"),
                self.field("page_size_selected"),
                self.field("total"),
            ),
        )
        self.register_field("can_pre", lambda: self.field("current_page") > 1)
        self.register_field(
            "can_next", lambda: self.field("current_page") < self.field("total_page")
        )
        page_setting_menu = MMenu(parent=self)
        self._display_label = MLabel()
        self._display_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self._change_page_size_button = MComboBox().small()
        self._change_page_size_button.set_menu(page_setting_menu)
        self._change_page_size_button.set_formatter(lambda x: f"{x} per page")

        self._pre_button = MToolButton().icon_only().svg("left_fill.svg").small()
        self._pre_button.clicked.connect(
            functools.partial(self._slot_change_current_page, -1)
        )
        self._next_button = MToolButton().small().icon_only().svg("right_fill.svg")
        self._next_button.clicked.connect(
            functools.partial(self._slot_change_current_page, 1)
        )
        self._current_page_spin_box = MSpinBox()
        self._current_page_spin_box.setMinimum(1)
        self._current_page_spin_box.set_dayu_size(MTheme().small)
        self._current_page_spin_box.valueChanged.connect(self._emit_page_changed)
        self._total_page_label = MLabel()

        self.bind("page_size_list", page_setting_menu, "data")
        self.bind(
            "page_size_selected", page_setting_menu, "value", signal="value_changed"
        )
        self.bind(
            "page_size_selected",
            self._change_page_size_button,
            "value",
            signal="value_changed",
        )
        self.bind(
            "current_page", self._current_page_spin_box, "value", signal="valueChanged"
        )
        self.bind("total_page", self._current_page_spin_box, "maximum")
        self.bind("total_page_text", self._total_page_label, "dayu_text")
        self.bind("display_text", self._display_label, "dayu_text")
        self.bind("can_pre", self._pre_button, "enabled")
        self.bind("can_next", self._next_button, "enabled")

        self._change_page_size_button.value_changed.connect(self._emit_page_changed)

        main_lay = QtWidgets.QHBoxLayout()
        main_lay.setContentsMargins(0, 0, 0, 0)
        main_lay.setSpacing(2)
        main_lay.addStretch()
        main_lay.addWidget(self._display_label)
        main_lay.addStretch()
        main_lay.addWidget(MLabel("|").secondary())
        main_lay.addWidget(self._change_page_size_button)
        main_lay.addWidget(MLabel("|").secondary())
        main_lay.addWidget(self._pre_button)
        main_lay.addWidget(MLabel("Page"))
        main_lay.addWidget(self._current_page_spin_box)
        main_lay.addWidget(MLabel("/"))
        main_lay.addWidget(self._total_page_label)
        main_lay.addWidget(self._next_button)
        self.setLayout(main_lay)

    def set_total(self, value: int):
        """Set page component total count."""
        self.set_field("total", value)
        self.set_field("current_page", 1)

    def _slot_change_current_page(self, offset: int):
        self.set_field("current_page", self.field("current_page") + offset)
        self._emit_page_changed()

    def set_page_config(self, data_list: list[int | dict[str, Any]]):
        """Set page component per page settings."""
        self.set_field(
            "page_size_list",
            [
                {"label": str(data), "value": data} if isinstance(data, int) else data
                for data in data_list
            ],
        )

    def _emit_page_changed(self):
        self.sig_page_changed.emit(
            self.field("page_size_selected"), self.field("current_page")
        )



from qtpy import QtWidgets

from qtui.dayu.button_group import MRadioButtonGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.drawer import MDrawer
from qtui.dayu.label import MLabel
from qtui.dayu.line_edit import MLineEdit
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MIcon, get_scale_factor
from qtui.dayu.spin_box import MDateEdit, MSpinBox


class DrawerExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MDrawer")
        self._init_ui()

    def _init_ui(self):
        scale_x, _ = get_scale_factor()
        self.button_grp = MRadioButtonGroup()
        self.button_grp.set_button_list(
            ["top", {"text": "right", "checked": True}, "bottom", "left"]
        )

        open_button_2 = MPushButton("Open").primary()
        open_button_2.clicked.connect(self.slot_open_button_2)
        placement_lay = QtWidgets.QHBoxLayout()
        placement_lay.addWidget(self.button_grp)
        placement_lay.addSpacing(int(20 * scale_x))
        placement_lay.addWidget(open_button_2)
        placement_lay.addStretch()

        new_account_button = MPushButton(
            text="New account", icon=MIcon("add_line.svg", "#fff")
        ).primary()
        new_account_button.clicked.connect(self.slot_new_account)
        new_account_lay = QtWidgets.QHBoxLayout()
        new_account_lay.addWidget(MLabel("Submit form in drawer"))
        new_account_lay.addWidget(new_account_button)
        new_account_lay.addStretch()

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Custom Placement"))
        main_lay.addLayout(placement_lay)
        main_lay.addWidget(MDivider("Submit form in drawer"))
        main_lay.addLayout(new_account_lay)

        main_lay.addWidget(MDivider("Preview drawer"))
        self.setLayout(main_lay)

    def slot_open_button(self):
        custom_widget = QtWidgets.QWidget()
        custom_lay = QtWidgets.QVBoxLayout()
        custom_lay.addWidget(MLabel("Some contents..."))
        custom_lay.addWidget(MLabel("Some contents..."))
        custom_lay.addWidget(MLabel("Some contents..."))
        custom_widget.setLayout(custom_lay)

        drawer = MDrawer("Basic Drawer", parent=self).left()
        scale_x, _ = get_scale_factor()
        drawer.setFixedWidth(int(300 * scale_x))
        drawer.set_widget(custom_widget)
        drawer.show()

    def slot_open_button_2(self):
        custom_widget = QtWidgets.QWidget()
        custom_lay = QtWidgets.QVBoxLayout()
        custom_lay.addWidget(MLabel("Some contents..."))
        custom_lay.addWidget(MLabel("Some contents..."))
        custom_lay.addWidget(MLabel("Some contents..."))
        custom_widget.setLayout(custom_lay)

        drawer = MDrawer("Basic Drawer", parent=self)
        drawer.set_dayu_position(
            self.button_grp.get_button_group().checkedButton().text()
        )

        scale_x, _ = get_scale_factor()
        drawer.setFixedWidth(int(300 * scale_x))
        drawer.set_widget(custom_widget)
        drawer.show()

    def slot_new_account(self):
        custom_widget = QtWidgets.QWidget()
        custom_lay = QtWidgets.QFormLayout()
        custom_lay.addRow("Name", MLineEdit())
        custom_lay.addRow("Age", MSpinBox())
        custom_lay.addRow("Birth", MDateEdit())
        custom_widget.setLayout(custom_lay)

        drawer = MDrawer("New account", parent=self)
        submit_button = MPushButton("Submit").primary()
        submit_button.clicked.connect(drawer.close)
        drawer.add_widget_to_bottom(MPushButton("Cancel"))
        drawer.add_widget_to_bottom(submit_button)
        scale_x, _ = get_scale_factor()
        drawer.setFixedWidth(int(300 * scale_x))
        drawer.set_widget(custom_widget)
        drawer.show()


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = DrawerExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtCore, QtWidgets

from qtui.dayu.browser import (
    MClickBrowserFilePushButton,
    MClickBrowserFileToolButton,
    MClickBrowserFolderPushButton,
    MClickBrowserFolderToolButton,
    MDragFileButton,
    MDragFolderButton,
)
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.qt import MIcon


class BrowserExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MBrowser...")
        self._init_ui()

    def _init_ui(self):
        browser_1 = MClickBrowserFilePushButton(
            text="Browser File PushButton"
        ).primary()
        browser_2 = MClickBrowserFolderPushButton(text="Browser Folder PushButton")
        browser_2.setIcon(MIcon("upload_line.svg"))
        browser_3 = MClickBrowserFilePushButton(
            text="Browser Multi Files", multiple=True
        ).primary()
        lay_1 = QtWidgets.QHBoxLayout()
        lay_1.addWidget(browser_1)
        lay_1.addWidget(browser_2)
        lay_1.addWidget(browser_3)

        browser_4 = MClickBrowserFileToolButton().huge()
        label_4 = MLabel()
        label_4.set_elide_mode(QtCore.Qt.TextElideMode.ElideMiddle)
        browser_4.sig_file_changed.connect(label_4.setText)

        browser_5 = MClickBrowserFolderToolButton().huge()
        label_5 = MLabel()
        label_5.set_elide_mode(QtCore.Qt.TextElideMode.ElideMiddle)
        browser_5.sig_folder_changed.connect(label_5.setText)

        lay_2 = QtWidgets.QHBoxLayout()
        lay_2.addWidget(label_4)
        lay_2.addWidget(browser_4)
        lay_2.addWidget(label_5)
        lay_2.addWidget(browser_5)

        browser_6 = MDragFileButton(text="Click or drag file here")
        browser_6.set_svg("attachment_line.svg")
        browser_6.set_dayu_filters([".egnx"])
        browser_6.set_dayu_multiple(True)
        label_6 = MLabel()
        label_6.set_elide_mode(QtCore.Qt.TextElideMode.ElideMiddle)
        browser_6.sig_file_changed.connect(label_6.setText)

        browser_7 = MDragFolderButton()
        browser_7.set_dayu_multiple(True)
        label_7 = MLabel()
        label_7.set_elide_mode(QtCore.Qt.TextElideMode.ElideRight)
        browser_7.sig_folder_changed.connect(label_7.setText)

        lay_3 = QtWidgets.QGridLayout()
        lay_3.addWidget(browser_6, 2, 0)
        lay_3.addWidget(browser_7, 2, 1)
        lay_3.addWidget(label_6, 3, 0)
        lay_3.addWidget(label_7, 3, 1)

        browser_8 = MDragFileButton(
            text="Click or drag media file here", multiple=False
        )
        browser_8.set_svg("media_line.svg")
        browser_8.set_dayu_filters([".mov", ".mp4"])
        browser_8_label = MLabel()
        browser_8_label.set_elide_mode(QtCore.Qt.TextElideMode.ElideRight)
        self.register_field("current_file", "")
        self.bind("current_file", browser_8, "dayu_path", signal="sig_file_changed")
        self.bind("current_file", browser_8_label, "text")

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("MClickBrowser*PushButton"))
        main_lay.addLayout(lay_1)
        main_lay.addWidget(MDivider("MClickBrowser*ToolButton"))
        main_lay.addLayout(lay_2)
        main_lay.addWidget(MDivider("MDragBrowser*ToolButton"))
        main_lay.addLayout(lay_3)
        main_lay.addWidget(MDivider("data bind"))
        main_lay.addWidget(browser_8)
        main_lay.addWidget(browser_8_label)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = BrowserExample()
        MTheme().apply(test)
        test.show()



import functools
from collections.abc import Callable
from typing import Any

from qtpy import QtWidgets

from qtui.dayu.button_group import MPushButtonGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.message import MMessage
from qtui.dayu.push_button import MPushButton


class MessageExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MMessage")
        self._init_ui()

    def _init_ui(self):
        button3 = MPushButton(text="Normal Message").primary()
        button4 = MPushButton(text="Success Message").success()
        button5 = MPushButton(text="Warning Message").warning()
        button6 = MPushButton(text="Error Message").danger()
        button3.clicked.connect(
            functools.partial(
                self.slot_show_message,
                MMessage.info,
                {"text": "This is a normal message"},
            )
        )
        button4.clicked.connect(
            functools.partial(
                self.slot_show_message,
                MMessage.success,
                {"text": "Congratulations, you have succeeded!"},
            )
        )
        button5.clicked.connect(
            functools.partial(
                self.slot_show_message, MMessage.warning, {"text": "I warned you!"}
            )
        )
        button6.clicked.connect(
            functools.partial(
                self.slot_show_message, MMessage.error, {"text": "You failed!"}
            )
        )

        sub_lay1 = QtWidgets.QHBoxLayout()
        sub_lay1.addWidget(button3)
        sub_lay1.addWidget(button4)
        sub_lay1.addWidget(button5)
        sub_lay1.addWidget(button6)

        button_duration = MPushButton(text="show 5s Message")
        button_duration.clicked.connect(
            functools.partial(
                self.slot_show_message,
                MMessage.info,
                {"text": "This message will be displayed for 5 seconds", "duration": 5},
            )
        )
        button_closable = MPushButton(text="closable Message")
        button_closable.clicked.connect(
            functools.partial(
                self.slot_show_message,
                MMessage.info,
                {"text": "The message can be manually closed", "closable": True},
            )
        )
        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different type"))
        main_lay.addLayout(sub_lay1)
        main_lay.addWidget(
            MLabel(
                "Different message types: normal, success, warning, error. Default 2 seconds after disappearance"  # noqa: E501
            )
        )
        main_lay.addWidget(MDivider("set duration"))
        main_lay.addWidget(button_duration)
        main_lay.addWidget(
            MLabel("Custom duration, set duration value in config, unit is seconds")
        )

        main_lay.addWidget(MDivider("set closable"))
        main_lay.addWidget(button_closable)
        main_lay.addWidget(
            MLabel("Set whether it can be closed, set closable to True in config")
        )

        button_grp = MPushButtonGroup()
        button_grp.set_button_list(
            [
                {
                    "text": "set duration to 1s",
                    "clicked": functools.partial(
                        self.slot_set_config, MMessage.config, {"duration": 1}
                    ),
                },
                {
                    "text": "set duration to 10s",
                    "clicked": functools.partial(
                        self.slot_set_config, MMessage.config, {"duration": 10}
                    ),
                },
                {
                    "text": "set top to 5",
                    "clicked": functools.partial(
                        self.slot_set_config, MMessage.config, {"top": 5}
                    ),
                },
                {
                    "text": "set top to 50",
                    "clicked": functools.partial(
                        self.slot_set_config, MMessage.config, {"top": 50}
                    ),
                },
            ]
        )
        loading_button = MPushButton("Display a loading indicator")
        loading_button.clicked.connect(self.slot_show_loading)
        main_lay.addWidget(MDivider("set global setting"))
        main_lay.addWidget(button_grp)
        main_lay.addWidget(
            MLabel(
                "Global setting default duration (default 2 seconds); top (distance from parent top, default 24px)"  # noqa: E501
            )
        )
        main_lay.addWidget(loading_button)

        main_lay.addStretch()
        self.setLayout(main_lay)

    def slot_show_message(self, func: Callable[..., Any], config: dict[str, Any]):
        func(parent=self, **config)

    def slot_set_config(self, func: Callable[..., Any], config: dict[str, Any]):
        func(**config)

    def slot_show_loading(self):
        msg = MMessage.loading("Loading...", parent=self)
        msg.sig_closed.connect(
            functools.partial(MMessage.success, "Loading success!", self)
        )


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = MessageExample()
        MTheme().apply(test)
        test.show()
